// 
// Copyright (c) 2010 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading.Tasks;

using Smdn.Collections;

namespace Smdn.Net.Pop3.Client {
  partial class PopClient {
    public string DownloadMessageAsText(long messageNumber, Encoding encoding)
    {
      return GetMessage(messageNumber).ReadAllText(encoding);
    }

    public Task<string> DownloadMessageAsTextAsync(long messageNumber, Encoding encoding)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => t.Result.ReadAllText(encoding));
    }

    public string DownloadFirstMessageAsText(Encoding encoding)
    {
      return GetFirstMessage().ReadAllText(encoding);
    }

    public Task<string> DownloadFirstMessageAsTextAsync(Encoding encoding)
    {
      return GetFirstMessageAsync().ContinueWith(t => t.Result.ReadAllText(encoding));
    }

    public string DownloadLastMessageAsText(Encoding encoding)
    {
      return GetLastMessage().ReadAllText(encoding);
    }

    public Task<string> DownloadLastMessageAsTextAsync(Encoding encoding)
    {
      return GetLastMessageAsync().ContinueWith(t => t.Result.ReadAllText(encoding));
    }

    private IReadOnlyList<TResult> DownloadAllMessagesCore<TResult>(Converter<PopMessage, TResult> selector)
    {
      return GetMessages(false).ConvertAll(selector);
    }

    private Task<IReadOnlyList<TResult>> DownloadAllMessagesAsyncCore<TResult>(Converter<PopMessage, TResult> selector)
    {
      return GetMessagesAsync(false).ContinueWith(t => (IReadOnlyList<TResult>)t.Result.ConvertAll(selector));
    }

    public IReadOnlyList<string> DownloadAllMessagesAsText(Encoding encoding)
    {
      return DownloadAllMessagesCore(message => message.ReadAllText(encoding));
    }

    public Task<IReadOnlyList<string>> DownloadAllMessagesAsTextAsync(Encoding encoding)
    {
      return DownloadAllMessagesAsyncCore(message => message.ReadAllText(encoding));
    }

    public byte[] DownloadMessageAsByteArray(long messageNumber)
    {
      return GetMessage(messageNumber).ReadAllBytes();
    }

    public Task<byte[]> DownloadMessageAsByteArrayAsync(long messageNumber)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => t.Result.ReadAllBytes());
    }

    public byte[] DownloadFirstMessageAsByteArray()
    {
      return GetFirstMessage().ReadAllBytes();
    }

    public Task<byte[]> DownloadFirstMessageAsByteArrayAsync()
    {
      return GetFirstMessageAsync().ContinueWith(t => t.Result.ReadAllBytes());
    }

    public byte[] DownloadLastMessageAsByteArray()
    {
      return GetLastMessage().ReadAllBytes();
    }

    public Task<byte[]> DownloadLastMessageAsByteArrayAsync()
    {
      return GetLastMessageAsync().ContinueWith(t => t.Result.ReadAllBytes());
    }

    public Stream DownloadMessageAsStream(long messageNumber)
    {
      return GetMessage(messageNumber).OpenRead();
    }

    public Task<Stream> DownloadMessageAsStreamAsync(long messageNumber)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => t.Result.OpenRead());
    }

    public Stream DownloadFirstMessageAsStream()
    {
      return GetFirstMessage().OpenRead();
    }

    public Task<Stream> DownloadFirstMessageAsStreamAsync()
    {
      return GetFirstMessageAsync().ContinueWith(t => t.Result.OpenRead());
    }

    public Stream DownloadLastMessageAsStream()
    {
      return GetLastMessage().OpenRead();
    }

    public Task<Stream> DownloadLastMessageAsStreamAsync()
    {
      return GetLastMessageAsync().ContinueWith(t => t.Result.OpenRead());
    }

    public TResult DownloadMessageAs<TResult>(long messageNumber, Converter<Stream, TResult> converter)
    {
      return GetMessage(messageNumber).ReadAs(converter);
    }

    public Task<TResult> DownloadMessageAsAsync<TResult>(long messageNumber, Converter<Stream, TResult> converter)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => t.Result.ReadAs(converter));
    }

    public TResult DownloadMessageAs<T, TResult>(long messageNumber, Func<Stream, T, TResult> read, T arg)
    {
      return GetMessage(messageNumber).ReadAs(read, arg);
    }

    public Task<TResult> DownloadMessageAsAsync<T, TResult>(long messageNumber, Func<Stream, T, TResult> read, T arg)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => t.Result.ReadAs(read, arg));
    }

    public TResult DownloadFirstMessageAs<TResult>(Converter<Stream, TResult> converter)
    {
      return GetFirstMessage().ReadAs(converter);
    }

    public Task<TResult> DownloadFirstMessageAsAsync<TResult>(Converter<Stream, TResult> converter)
    {
      return GetFirstMessageAsync().ContinueWith(t => t.Result.ReadAs(converter));
    }

    public TResult DownloadFirstMessageAs<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return GetFirstMessage().ReadAs(read, arg);
    }

    public Task<TResult> DownloadFirstMessageAsAsync<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return GetFirstMessageAsync().ContinueWith(t => t.Result.ReadAs(read, arg));
    }

    public TResult DownloadLastMessageAs<TResult>(Converter<Stream, TResult> converter)
    {
      return GetLastMessage().ReadAs(converter);
    }

    public Task<TResult> DownloadLastMessageAsAsync<TResult>(Converter<Stream, TResult> converter)
    {
      return GetLastMessageAsync().ContinueWith(t => t.Result.ReadAs(converter));
    }

    public TResult DownloadLastMessageAs<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return GetLastMessage().ReadAs(read, arg);
    }

    public Task<TResult> DownloadLastMessageAsAsync<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return GetLastMessageAsync().ContinueWith(t => t.Result.ReadAs(read, arg));
    }

    public IReadOnlyList<TResult> DownloadAllMessagesAs<TResult>(Converter<Stream, TResult> converter)
    {
      return DownloadAllMessagesCore(message => message.ReadAs(converter));
    }

    public Task<IReadOnlyList<TResult>> DownloadAllMessagesAsAsync<TResult>(Converter<Stream, TResult> converter)
    {
      return DownloadAllMessagesAsyncCore(message => message.ReadAs(converter));
    }

    public IReadOnlyList<TResult> DownloadAllMessagesAs<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return DownloadAllMessagesCore(message => message.ReadAs(read, arg));
    }

    public Task<IReadOnlyList<TResult>> DownloadAllMessagesAsAsync<T, TResult>(Func<Stream, T, TResult> read, T arg)
    {
      return DownloadAllMessagesAsyncCore(message => message.ReadAs(read, arg));
    }

    public FileInfo DownloadMessageToFile(long messageNumber, string path)
    {
      var file = new FileInfo(path);

      GetMessage(messageNumber).Body.Save(file, PopMessage.EntireMessage);

      return file;
    }

    public Task<FileInfo> DownloadMessageToFileAsync(long messageNumber, string path)
    {
      return GetMessageAsync(messageNumber).ContinueWith(t => {
        var file = new FileInfo(path);

        t.Result.Body.Save(file, PopMessage.EntireMessage);

        return file;
      });
    }

    public FileInfo DownloadFirstMessageToFile(string path)
    {
      var file = new FileInfo(path);

      GetFirstMessage().Body.Save(file, PopMessage.EntireMessage);

      return file;
    }

    public Task<FileInfo> DownloadFirstMessageToFileAsync(string path)
    {
      return GetFirstMessageAsync().ContinueWith(t => {
        var file = new FileInfo(path);

        t.Result.Body.Save(file, PopMessage.EntireMessage);

        return file;
      });
    }

    public FileInfo DownloadLastMessageToFile(string path)
    {
      var file = new FileInfo(path);

      GetLastMessage().Body.Save(file, PopMessage.EntireMessage);

      return file;
    }

    public Task<FileInfo> DownloadLastMessageToFileAsync(string path)
    {
      return GetLastMessageAsync().ContinueWith(t => {
        var file = new FileInfo(path);

        t.Result.Body.Save(file, PopMessage.EntireMessage);

        return file;
      });
    }

    public IReadOnlyList<FileInfo> DownloadAllMessagesToDirectory(string directory)
    {
      if (directory == null)
        throw new ArgumentNullException(nameof(directory));
      if (directory.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString(nameof(directory));

      ThrowIfNotConnected();

      Directory.CreateDirectory(directory);

      return DownloadAllMessagesToFiles(msg => Path.Combine(directory, string.Format("{0}.eml", msg.MessageNumber)));
    }

    public Task<IReadOnlyList<FileInfo>> DownloadAllMessagesToDirectoryAsync(string directory)
    {
      if (directory == null)
        throw new ArgumentNullException(nameof(directory));
      if (directory.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString(nameof(directory));

      ThrowIfNotConnected();

      Directory.CreateDirectory(directory);

      return DownloadAllMessagesToFilesAsync(msg => Path.Combine(directory, string.Format("{0}.eml", msg.MessageNumber)));
    }

    public IReadOnlyList<FileInfo> DownloadAllMessagesToFiles(Func<PopMessage, string> selectFileName)
    {
      if (selectFileName == null)
        throw new ArgumentNullException(nameof(selectFileName));

      return DownloadAllMessagesCore(message => {
        var file = new FileInfo(selectFileName(message));

        message.Body.Save(file, PopMessage.EntireMessage);

        return file;
      });
    }

    public Task<IReadOnlyList<FileInfo>> DownloadAllMessagesToFilesAsync(Func<PopMessage, string> selectFileName)
    {
      if (selectFileName == null)
        throw new ArgumentNullException(nameof(selectFileName));

      return DownloadAllMessagesAsyncCore(message => {
        var file = new FileInfo(selectFileName(message));

        message.Body.Save(file, PopMessage.EntireMessage);

        return file;
      });
    }
  }
}
