// 
// Copyright (c) 2010 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Runtime.ExceptionServices;
using System.Threading;
using System.Threading.Tasks;

using Smdn.Net.Pop3.Protocol.Transmission;
using Smdn.Net.Pop3.Protocol.Transmission.Transaction;

using Trace = Smdn.Net.Pop3.Client.Trace;

namespace Smdn.Net.Pop3.Protocol.Session {
  partial class PopSession {
    /*
     * methods for processing transaction
     */
    private PopCommandResult ProcessTransaction(IPopTransaction t)
    {
      PreProcessTransaction(t, handlesIncapableAsException);

      try {
        if (!Task.Run(() => ProcessTransactionInternal(t)).Wait(transactionTimeout/* do not refer property TransactionTimeout to avoid throwing ObjectDisposedException */)) {
          CloseConnection();
          throw new TimeoutException(string.Format("transaction timeout ({0})", t.GetType().FullName));
        }
      }
      catch (AggregateException ex) {
        ExceptionDispatchInfo.Capture(ex.InnerException).Throw();
      }

      return t.Result;
    }

    private void ProcessTransactionInternal(IPopTransaction t)
    {
      lock (transactionLockObject) {
        Trace.LogRequest(t);

        try {
          t.Process();
        }
        catch {
          CloseConnection();
          throw;
        }

        Trace.LogResponse(t);
      }
    }

    private void PreProcessTransaction(IPopTransaction t, bool exceptionIfIncapable)
    {
      if (IsTransactionProceeding)
        throw new InvalidOperationException("another transaction is proceeding");

      // check capability
      if (!exceptionIfIncapable)
        return;

      if (t is IPopExtension extension)
        IsCapable(extension, throwIfIncapable: true);

      /*
      foreach (var arg in t.Request.Arguments.Values) {
        if (arg is IPopExtension)
          CheckServerCapability(arg as IPopExtension);
      }
      */
    }
  }
}
