// 
// Copyright (c) 2019 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.MessageAccessProtocols.Strings;
using Smdn.Net.Pop3.Protocol.Strings;

namespace Smdn.Net.Pop3.Protocol {
  public static class PopCapabilitySetExtensions {
    public static bool IsCapable(this IReadOnlyStringSet<PopCapability> capabilities, PopEnumValue feature)
    {
      if (feature == null)
        throw new ArgumentNullException(nameof(feature));

      if (feature is IPopExtension extension)
        return IsCapable(capabilities, extension.RequiredCapability);

      return true;
    }

    internal static bool IsCapable(this IReadOnlyStringSet<PopCapability> capabilities, IPopExtension extension)
    {
      if (capabilities == null)
        throw new ArgumentNullException(nameof(capabilities));

      if (extension.RequiredCapability == null)
        return true;

      return IsCapable(capabilities, extension.RequiredCapability);
    }

    public static bool IsCapable(this IReadOnlyStringSet<PopCapability> capabilities, PopCapability requiredCapability)
    {
      if (capabilities == null)
        throw new ArgumentNullException(nameof(capabilities));
      if (requiredCapability == null)
        throw new ArgumentNullException(nameof(requiredCapability));

      var capability = FindByTag(capabilities, requiredCapability);

      if (capability == null)
        return false;

      return capability.ContainsAllArguments(requiredCapability.Arguments);
    }

    public static PopCapability FindByTag(this IReadOnlyStringSet<PopCapability> capabilities, PopCapability capability)
    {
      if (capabilities == null)
        throw new ArgumentNullException(nameof(capabilities));
      if (capability == null)
        throw new ArgumentNullException(nameof(capability));

      return FindByTag(capabilities, capability.Tag);
    }

    public static PopCapability FindByTag(this IReadOnlyStringSet<PopCapability> capabilities, string tag)
    {
      if (capabilities == null)
        throw new ArgumentNullException(nameof(capabilities));
      if (tag == null)
        throw new ArgumentNullException(nameof(tag));

      foreach (var capability in capabilities) {
        if (string.Equals(capability.Tag, tag, StringComparison.OrdinalIgnoreCase))
          return capability;
      }

      return null;
    }
  }
}
