using System;
using NUnit.Framework;

namespace Smdn.Net.Pop3.Protocol {
  [TestFixture]
  public class PopAuthenticationMechanismTests {
    [Test]
    public void TestEquals()
    {
      var x = PopAuthenticationMechanism.Apop;

      Assert.IsTrue(x.Equals(PopAuthenticationMechanism.Apop));
      Assert.IsTrue(x.Equals(new PopAuthenticationMechanism("+apop")));
      Assert.IsTrue(x.Equals((object)"+APOP"));
      Assert.IsTrue(x.Equals((object)"+apop"));
      Assert.IsTrue(x.Equals((object)PopAuthenticationMechanism.Apop));
      Assert.IsTrue(x.Equals("+APOP"));
      Assert.IsTrue(x.Equals("+apop"));
      Assert.IsTrue(x.Equals(PopAuthenticationMechanism.Apop.Value));

      Assert.IsFalse(x.Equals(PopAuthenticationMechanism.Anonymous));
      Assert.IsFalse(x.Equals(new PopCapability("+APOP")));
      Assert.IsFalse(x.Equals((object)new PopCapability("+APOP")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = PopAuthenticationMechanism.Apop;

      Assert.IsTrue(x == PopAuthenticationMechanism.Apop);
      Assert.IsTrue(x == new PopAuthenticationMechanism("+apop"));
      Assert.IsFalse(x != PopAuthenticationMechanism.Apop);
      Assert.IsFalse(x != new PopAuthenticationMechanism("+apop"));

      Assert.IsFalse(x == PopAuthenticationMechanism.Anonymous);
      Assert.IsFalse(x == new PopAuthenticationMechanism("anonymous"));
      Assert.IsTrue(x != PopAuthenticationMechanism.Anonymous);
      Assert.IsTrue(x != new PopAuthenticationMechanism("anonymous"));
    }

    [Test]
    public void TestConstructor()
    {
      Assert.Throws<ArgumentNullException>(() => new PopAuthenticationMechanism(null));
      Assert.Throws<ArgumentException>(() => new PopAuthenticationMechanism(string.Empty));
    }

    [Test]
    public void TestValue()
    {
      var authType = new PopAuthenticationMechanism("X-AUTH");

      Assert.AreEqual("X-AUTH", authType.Value);
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("+APOP", PopAuthenticationMechanism.Apop.ToString());
      Assert.AreEqual("+apop", new PopAuthenticationMechanism("+apop").ToString());
      Assert.AreEqual("ANONYMOUS", PopAuthenticationMechanism.Anonymous.ToString());
    }
  }
}
