using System;
using NUnit.Framework;

namespace Smdn.Net.Pop3.Protocol {
  [TestFixture]
  public class PopResponseCodeTests {
    [Test]
    public void TestEquals()
    {
      var x = PopResponseCode.InUse;

      Assert.IsTrue(x.Equals(PopResponseCode.InUse));
      Assert.IsTrue(x.Equals((object)"IN-USE"));
      Assert.IsTrue(x.Equals((object)"in-use"));
      Assert.IsTrue(x.Equals((object)PopResponseCode.InUse));
      Assert.IsTrue(x.Equals("IN-USE"));
      Assert.IsTrue(x.Equals("in-use"));
      Assert.IsTrue(x.Equals(PopResponseCode.InUse.Value));

      Assert.IsFalse(x.Equals(PopResponseCode.Auth));
      Assert.IsFalse(x.Equals(new PopCapability("IN-USE")));
      Assert.IsFalse(x.Equals((object)new PopCapability("in-use")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = PopResponseCode.InUse;

      Assert.IsTrue(x == PopResponseCode.InUse);
      Assert.IsFalse(x != PopResponseCode.InUse);

      Assert.IsFalse(x == PopResponseCode.Auth);
      Assert.IsTrue(x != PopResponseCode.Auth);
    }

    [Test]
    public void TestValue()
    {
      Assert.AreEqual("AUTH", PopResponseCode.Auth.Value);
      Assert.AreEqual("SYS/PERM", PopResponseCode.SysPerm.Value);
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("AUTH", PopResponseCode.Auth.ToString());
      Assert.AreEqual("SYS/PERM", PopResponseCode.SysPerm.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(PopResponseCode.Auth, delegate(PopResponseCode deserialized) {
        Assert.AreNotSame(PopResponseCode.Auth, deserialized);
        Assert.AreEqual(PopResponseCode.Auth, deserialized);
      });

      TestUtils.Assert.IsSerializableBinaryFormat(PopResponseCode.SysPerm, delegate (PopResponseCode deserialized) {
        Assert.AreNotSame(PopResponseCode.SysPerm, deserialized);
        Assert.AreEqual(PopResponseCode.SysPerm, deserialized);
      });
    }
  }
}

